@@ -156,6 +156,7 @@ typedef enum {
    SFLAGS_SURFACE = 0, SFLAGS_UNDERGROUND, SFLAGS_ENTRANCE, SFLAGS_EXPORTED,
    SFLAGS_FIXED, SFLAGS_ANON, SFLAGS_WALL,
    /* These values don't need to match img.h, but mustn't clash. */
+   SFLAGS_HANGING = 10,
    SFLAGS_USED = 11,
    SFLAGS_SOLVED = 12, SFLAGS_SUSPECTTYPO = 13, SFLAGS_SURVEY = 14, SFLAGS_PREFIX_ENTERED = 15
 } sflags;
@@ -199,7 +200,11 @@ typedef enum {
    CompassDATLeft, CompassDATRight, CompassDATUp, CompassDATDown,
    CompassDATFlags,
 
-   WallsSRVFr, WallsSRVTo, WallsSRVTape, WallsSRVComp, WallsSRVClino
+   WallsSRVFr, WallsSRVTo, WallsSRVTape, WallsSRVComp, WallsSRVClino,
+   // Optional pair of readings giving heights above stations on CT surveys.
+   WallsSRVHeights,
+   // Optional delimited LRUD and variance overrides.
+   WallsSRVExtras
 } reading;
 
 /* if IgnoreAll is >= 32, the compiler will choke on this */
@@ -294,6 +299,12 @@ typedef struct Node {
    struct Prefix *name;
    struct Link *leg[3];
    struct Node *prev, *next;
+   // Used in netartic.c to identify unconnected components and articulation
+   // points within components.
+   //
+   // Used in matrix.c to record the matrix row corresponding to this node
+   // or -1 for nodes already fixed (more than one node may map to the same
+   // row).
    long colour;
 } node;
 
@@ -352,7 +363,9 @@ typedef struct Settings {
    real sc[Q_MAC];
    real units[Q_MAC];
    const reading *ordering;
+   long begin_lpos; /* File offset for start of BEGIN line */
    int begin_lineno; /* 0 means no block started in this file */
+   int begin_col; /* Column of prefix in BEGIN line (or 0 if none) */
    int flags;
    char* proj_str;
    /* Location at which we calculate the declination if
@@ -374,6 +387,12 @@ typedef struct Settings {
    char* dec_context;
    /* Grid convergence in radians. */
    real convergence;
+   /* Input grid convergence in radians. */
+   real input_convergence;
+   /* Rotation from North for `*data cartesian`. */
+   real cartesian_rotation;
+   /* Which North to use for `*data cartesian`. */
+   enum { TRUE_NORTH, GRID_NORTH, MAGNETIC_NORTH } cartesian_north;
    meta_data * meta;
 } settings;
 
@@ -390,6 +409,7 @@ extern string survey_title;
 
 extern bool fExplicitTitle;
 extern long cLegs, cStns, cComponents;
+extern bool hanging_surveys;
 extern FILE *fhErrStat;
 extern img *pimg;
 extern real totadj, total, totplan, totvert;
