@@ -63,11 +63,22 @@ void copy_on_write_meta(settings *s);
 
 extern string token;
 extern string uctoken;
+
+/* Read legacy token (letters only).  This only exists so we can keep reading
+ * old data files which (presumably accidentally) are missing blanks between
+ * a token and a number which follows.  Use get_token() in new code.
+ */
+void get_token_legacy(void);
+void get_token_legacy_no_blanks(void);
+
+// Issue warning if token not empty and ch not BLANK, COMM or EOL.
+void do_legacy_token_warning(void);
+
+// Read a token, comprised of a letter followed by contiguous alphanumerics.
 void get_token(void);
 void get_token_no_blanks(void);
-// Read a token as defined in Walls format.
-void get_token_walls(void);
 
+// Read up to the next BLANK, COMM or end of line.
 void get_word(void);
 
 typedef struct { const char *sz; int tok; } sztok;
