@@ -4,7 +4,7 @@
 //  Core drawing code for Aven.
 //
 //  Copyright (C) 2000-2001,2002,2005 Mark R. Shinwell.
-//  Copyright (C) 2001-2004,2005,2006,2007,2010,2011,2012,2013,2014,2015,2016,2017,2018 Olly Betts
+//  Copyright (C) 2001-2024 Olly Betts
 //  Copyright (C) 2005 Martin Green
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -52,13 +52,13 @@ class MovieMaker;
 
 class PresentationMark : public Point {
   public:
-    Double angle, tilt_angle;
-    Double scale;
-    Double time;
+    double angle, tilt_angle;
+    double scale;
+    double time;
     PresentationMark() : Point(), angle(0), tilt_angle(0), scale(0), time(0)
 	{ }
-    PresentationMark(const Vector3 & v, Double angle_, Double tilt_angle_,
-		     Double scale_, Double time_ = 0)
+    PresentationMark(const Vector3 & v, double angle_, double tilt_angle_,
+		     double scale_, double time_ = 0)
 	: Point(v), angle(angle_), tilt_angle(tilt_angle_), scale(scale_),
 	  time(time_)
 	{ }
@@ -127,8 +127,8 @@ struct Split {
 const int MAX_FRAMERATE = 50;
 
 class GfxCore : public GLACanvas {
-    Double m_Scale;
-    Double initial_scale;
+    double m_Scale;
+    double initial_scale;
     int m_ScaleBarWidth;
 
     typedef enum {
@@ -150,6 +150,7 @@ class GfxCore : public GLACanvas {
 	LIST_GRID,
 	LIST_SHADOW,
 	LIST_TERRAIN,
+	LIST_OVERLAYS,
 	LIST_LIMIT_ // Leave this last.
     } drawing_list;
 
@@ -186,10 +187,10 @@ private:
     char* m_LabelGrid;
     MainFrm* m_Parent;
     bool m_DoneFirstShow;
-    Double m_TiltAngle;
-    Double m_PanAngle;
+    double m_TiltAngle;
+    double m_PanAngle;
     bool m_Rotating;
-    Double m_RotationStep;
+    double m_RotationStep;
     int m_SwitchingTo;
     bool m_Crosses;
     bool m_Legs;
@@ -235,8 +236,8 @@ private:
     wxStopWatch timer;
     long base_tilt_time;
     long base_pan_time;
-    Double base_tilt;
-    Double base_pan;
+    double base_tilt;
+    double base_pan;
 
     GLAPen m_Pens[NUM_COLOUR_BANDS + 1];
 
@@ -274,24 +275,24 @@ private:
     long last_time;
     size_t n_tris;
 
-    void PlaceVertexWithColour(const Vector3 &v, Double factor = 1.0);
+    void PlaceVertexWithColour(const Vector3 &v, double factor = 1.0);
     void PlaceVertexWithColour(const Vector3 & v,
 			       glaTexCoord tex_x, glaTexCoord tex_y,
-			       Double factor);
-    void SetDepthColour(Double z, Double factor);
-    void PlaceVertexWithDepthColour(const Vector3 & v, Double factor = 1.0);
+			       double factor);
+    void SetDepthColour(double z, double factor);
+    void PlaceVertexWithDepthColour(const Vector3 & v, double factor = 1.0);
     void PlaceVertexWithDepthColour(const Vector3 & v,
 				    glaTexCoord tex_x, glaTexCoord tex_y,
-				    Double factor);
+				    double factor);
 
-    void SetColourFrom01(double how_far, Double factor);
+    void SetColourFrom01(double how_far, double factor);
 
-    void SetColourFromDate(int date, Double factor);
-    void SetColourFromError(double E, Double factor);
-    void SetColourFromGradient(double angle, Double factor);
-    void SetColourFromLength(double len, Double factor);
+    void SetColourFromDate(int date, double factor);
+    void SetColourFromError(double E, double factor);
+    void SetColourFromGradient(double angle, double factor);
+    void SetColourFromLength(double len, double factor);
     void SetColourFromSurvey(const wxString& survey);
-    void SetColourFromSurveyStation(const wxString& survey, Double factor);
+    void SetColourFromSurveyStation(const wxString& survey, double factor);
 
     int GetClinoOffset() const;
     void DrawTick(int angle_cw);
@@ -386,9 +387,9 @@ public:
     void CentreOn(const Point &p);
 
     void TranslateCave(int dx, int dy);
-    void TiltCave(Double tilt_angle);
-    void TurnCave(Double angle);
-    void TurnCaveTo(Double angle);
+    void TiltCave(double tilt_angle);
+    void TurnCave(double angle);
+    void TurnCaveTo(double angle);
 
     void OnPaint(wxPaintEvent&);
     void OnSize(wxSizeEvent& event);
@@ -441,7 +442,7 @@ public:
     void SwitchToElevation();
     void SwitchToPlan();
 
-    void SetViewTo(Double xmin, Double xmax, Double ymin, Double ymax, Double zmin, Double zmax);
+    void SetViewTo(double xmin, double xmax, double ymin, double ymax, double zmin, double zmax);
 
     double GetCompassValue() const { return m_PanAngle; }
     bool ShowingPlan() const;
@@ -460,7 +461,7 @@ public:
     bool HasDateInformation() const;
 
     double GetScale() const { return m_Scale; }
-    void SetScale(Double scale);
+    void SetScale(double scale);
 
     bool ShowingStationNames() const { return m_Names; }
     bool ShowingOverlappingNames() const { return m_OverlappingNames; }
@@ -583,14 +584,14 @@ public:
 
     void SplitLineAcrossBands(int band, int band2,
 			      const Vector3 &p, const Vector3 &q,
-			      Double factor = 1.0);
+			      double factor = 1.0);
     void SplitPolyAcrossBands(vector<vector<Split>>& splits,
 			      int band, int band2,
 			      const Vector3 &p, const Vector3 &q,
 			      glaTexCoord ptx, glaTexCoord pty,
 			      glaTexCoord w, glaTexCoord h);
-    int GetDepthColour(Double z) const;
-    Double GetDepthBoundaryBetweenBands(int a, int b) const;
+    int GetDepthColour(double z) const;
+    double GetDepthBoundaryBetweenBands(int a, int b) const;
     void AddPolyline(const traverse & centreline);
     void AddPolylineDepth(const traverse & centreline);
     void AddPolylineDate(const traverse & centreline);
@@ -654,11 +655,17 @@ public:
 
     void ZoomBoxGo();
 
+    void DrawOverlays();
+
     void parse_hgt_filename(const wxString & lc_name);
     size_t parse_hdr(wxInputStream & is, unsigned long & skipbytes);
     bool read_bil(wxInputStream & is, size_t size, unsigned long skipbytes);
     bool LoadDEM(const wxString & file);
 
+    void InvalidateOverlays() {
+	InvalidateList(LIST_OVERLAYS);
+    }
+
 private:
     DECLARE_EVENT_TABLE()
 };
