@@ -2,7 +2,7 @@
  * Export from Aven as GPX.
  */
 /* Copyright (C) 2012 Olaf Kähler
- * Copyright (C) 2012,2013,2014,2015,2016 Olly Betts
+ * Copyright (C) 2012-2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -181,7 +181,7 @@ GPX::line(const img_point *p1, const img_point *p, unsigned /*flags*/, bool fPen
 }
 
 void
-GPX::label(const img_point *p, const wxString& str, bool /*fSurface*/, int type)
+GPX::label(const img_point *p, const wxString& str, int /*sflags*/, int type)
 {
     const char* s = str.utf8_str();
     PJ_COORD coord{{p->x, p->y, p->z, HUGE_VAL}};
