@@ -57,8 +57,6 @@ static void sor(real *M, real *B, long n);
 	      /* +(Y>X?0*printf("row<col (line %d)\n",__LINE__):0) */
 /*#define M_(X, Y) ((real *)M)[((((OSSIZE_T)(Y)) * ((Y) + 1)) >> 1) + (X)]*/
 
-static int find_stn_in_tab(node *stn);
-static int add_stn_to_tab(node *stn);
 static void build_matrix(node *list);
 
 static long n_stn_tab;
@@ -71,24 +69,36 @@ solve_matrix(node *list)
    node *stn;
    long n = 0;
    FOR_EACH_STN(stn, list) {
-      if (!fixed(stn)) n++;
+      if (!fixed(stn))
+	  n++;
    }
    if (n == 0) return;
 
-   /* we just need n to be a reasonable estimate >= the number
-    * of stations left after reduction. If memory is
-    * plentiful, we can be crass.
+   /* We need to allocate stn_tab with one entry per unfixed cluster of equated
+    * stations, but it's much simpler to count the number of unfixed nodes.
+    * This will over-count stations with more than 3 legs and equated stations
+    * but in a typical survey that's a small minority of stations.
     */
    stn_tab = osmalloc((OSSIZE_T)(n * ossizeof(pos*)));
    n_stn_tab = 0;
 
+   /* We store the stn_tab index in stn->colour for quick and easy lookup in
+    * build_matrix().
+    */
    FOR_EACH_STN(stn, list) {
-      if (!fixed(stn)) add_stn_to_tab(stn);
-   }
-
-   if (n_stn_tab < n) {
-      /* release unused entries in stn_tab */
-      stn_tab = osrealloc(stn_tab, n_stn_tab * ossizeof(pos*));
+      if (!fixed(stn)) {
+	  int i;
+	  pos *p = stn->name->pos;
+	  for (i = 0; i < n_stn_tab; i++) {
+	      if (stn_tab[i] == p)
+		  break;
+	  }
+	  if (i == n_stn_tab)
+	      stn_tab[n_stn_tab++] = p;
+	  stn->colour = i;
+      } else {
+	  stn->colour = -1;
+      }
    }
 
    build_matrix(list);
@@ -112,18 +122,10 @@ solve_matrix(node *list)
 static void
 build_matrix(node *list)
 {
-   real *M;
-   real *B;
-   int dim;
-
-   if (n_stn_tab == 0) {
-      if (!fQuiet)
-	 puts(msg(/*Network solved by reduction - no simultaneous equations to solve.*/74));
-      return;
-   }
+   SVX_ASSERT(n_stn_tab > 0);
    /* (OSSIZE_T) cast may be needed if n_stn_tab>=181 */
-   M = osmalloc((OSSIZE_T)((((OSSIZE_T)n_stn_tab * FACTOR * (n_stn_tab * FACTOR + 1)) >> 1)) * ossizeof(real));
-   B = osmalloc((OSSIZE_T)(n_stn_tab * FACTOR * ossizeof(real)));
+   real *M = osmalloc((OSSIZE_T)((((OSSIZE_T)n_stn_tab * FACTOR * (n_stn_tab * FACTOR + 1)) >> 1)) * ossizeof(real));
+   real *B = osmalloc((OSSIZE_T)(n_stn_tab * FACTOR * ossizeof(real)));
 
    if (!fQuiet) {
       if (n_stn_tab == 1)
@@ -133,9 +135,9 @@ build_matrix(node *list)
    }
 
 #ifdef NO_COVARIANCES
-   dim = 2;
+   int dim = 2;
 #else
-   dim = 0; /* fudge next loop for now */
+   int dim = 0; /* fudge next loop for now */
 #endif
    for ( ; dim >= 0; dim--) {
       node *stn;
@@ -150,15 +152,24 @@ build_matrix(node *list)
 	 for (row = 0; row < end; row++) M[row] = (real)0.0;
       }
 
-      /* Construct matrix - Go thru' stn list & add all forward legs between
-       * two unfixed stations to M (so each leg goes on exactly once).
+      /* Construct matrix by going through the stn list.
        *
        * All legs between two fixed stations can be ignored here.
        *
-       * All legs between a fixed and an unfixed station are then considered
-       * from the unfixed end (if we consider them from the fixed end we'd
-       * need to somehow detect when we're at a fixed point cut line and work
-       * out which side we're dealing with at this time. */
+       * Other legs we want to add exactly once to M.  To achieve this we
+       * wan to:
+       *
+       * - add forward legs between two unfixed stations,
+       *
+       * - add legs from unfixed stations to fixed stations (we do them from
+       *   the unfixed end so we don't need to detect when we're at a fixed
+       *   point cut line and determine which side we're currently dealing
+       *   with).
+       *
+       * To implement this, we only look at legs from unfixed stations and add
+       * a leg if to a fixed station, or to an unfixed station and it's a
+       * forward leg.
+       */
       FOR_EACH_STN(stn, list) {
 #ifdef NO_COVARIANCES
 	 real e;
@@ -166,15 +177,13 @@ build_matrix(node *list)
 	 svar e;
 	 delta a;
 #endif
-	 int f, t;
-	 int dirn;
 #if DEBUG_MATRIX_BUILD
 	 print_prefix(stn->name);
 	 printf(" used: %d colour %ld\n",
 		(!!stn->leg[2]) << 2 | (!!stn -> leg[1]) << 1 | (!!stn->leg[0]),
 		stn->colour);
 
-	 for (dirn = 0; dirn <= 2 && stn->leg[dirn]; dirn++) {
+	 for (int dirn = 0; dirn <= 2 && stn->leg[dirn]; dirn++) {
 #ifdef NO_COVARIANCES
 	    printf("Leg %d, vx=%f, reverse=%d, to ", dirn,
 		   stn->leg[dirn]->v[0], stn->leg[dirn]->l.reverse);
@@ -189,8 +198,8 @@ build_matrix(node *list)
 #endif /* DEBUG_MATRIX_BUILD */
 
 	 if (!fixed(stn)) {
-	    f = find_stn_in_tab(stn);
-	    for (dirn = 0; dirn <= 2 && stn->leg[dirn]; dirn++) {
+	    int f = stn->colour;
+	    for (int dirn = 0; dirn <= 2 && stn->leg[dirn]; dirn++) {
 	       linkfor *leg = stn->leg[dirn];
 	       node *to = leg->l.to;
 	       if (fixed(to)) {
@@ -211,15 +220,14 @@ build_matrix(node *list)
 		  }
 #else
 		  if (invert_svar(&e, &leg->v)) {
-		     delta b;
-		     int i;
 		     if (fRev) {
 			adddd(&a, &POSD(to), &leg->d);
 		     } else {
 			subdd(&a, &POSD(to), &leg->d);
 		     }
+		     delta b;
 		     mulsd(&b, &e, &a);
-		     for (i = 0; i < 3; i++) {
+		     for (int i = 0; i < 3; i++) {
 			M(f * FACTOR + i, f * FACTOR + i) += e[i];
 			B[f * FACTOR + i] += b[i];
 		     }
@@ -230,7 +238,7 @@ build_matrix(node *list)
 #endif
 	       } else if (data_here(leg)) {
 		  /* forward leg, unfixed -> unfixed */
-		  t = find_stn_in_tab(to);
+		  int t = to->colour;
 #if DEBUG_MATRIX
 		  printf("Leg %d to %d, var %f, delta %f\n", f, t, e,
 			 leg->d[dim]);
@@ -239,20 +247,18 @@ build_matrix(node *list)
 #ifdef NO_COVARIANCES
 		  e = leg->v[dim];
 		  if (t != f && e != (real)0.0) {
-		     real a;
 		     e = ((real)1.0) / e;
 		     M(f,f) += e;
 		     M(t,t) += e;
 		     if (f < t) M(t,f) -= e; else M(f,t) -= e;
-		     a = e * leg->d[dim];
+		     real a = e * leg->d[dim];
 		     B[f] -= a;
 		     B[t] += a;
 		  }
 #else
 		  if (t != f && invert_svar(&e, &leg->v)) {
-		     int i;
 		     mulsd(&a, &e, &leg->d);
-		     for (i = 0; i < 3; i++) {
+		     for (int i = 0; i < 3; i++) {
 			M(f * FACTOR + i, f * FACTOR + i) += e[i];
 			M(t * FACTOR + i, t * FACTOR + i) += e[i];
 			if (f < t)
@@ -303,67 +309,37 @@ build_matrix(node *list)
 	 choleski(M, B, n_stn_tab * FACTOR);
 
       {
-	 int m;
-	 for (m = (int)(n_stn_tab - 1); m >= 0; m--) {
+	 for (int m = (int)(n_stn_tab - 1); m >= 0; m--) {
 #ifdef NO_COVARIANCES
 	    stn_tab[m]->p[dim] = B[m];
+# if !EXPLICIT_FIXED_FLAG
 	    if (dim == 0) {
 	       SVX_ASSERT2(pos_fixed(stn_tab[m]),
 		       "setting station coordinates didn't mark pos as fixed");
 	    }
+# endif
 #else
-	    int i;
-	    for (i = 0; i < 3; i++) {
+	    for (int i = 0; i < 3; i++) {
 	       stn_tab[m]->p[i] = B[m * FACTOR + i];
 	    }
+# if !EXPLICIT_FIXED_FLAG
 	    SVX_ASSERT2(pos_fixed(stn_tab[m]),
 		    "setting station coordinates didn't mark pos as fixed");
+# endif
+#endif
+#if EXPLICIT_FIXED_FLAG && !defined NO_COVARIANCES
+	    fixpos(stn_tab[m]);
 #endif
 	 }
-#if EXPLICIT_FIXED_FLAG
-	 for (m = n_stn_tab - 1; m >= 0; m--) fixpos(stn_tab[m]);
-#endif
       }
    }
+#if EXPLICIT_FIXED_FLAG && defined NO_COVARIANCES
+   for (int m = n_stn_tab - 1; m >= 0; m--) fixpos(stn_tab[m]);
+#endif
    osfree(B);
    osfree(M);
 }
 
-static int
-find_stn_in_tab(node *stn)
-{
-   int i = 0;
-   pos *p = stn->name->pos;
-   while (stn_tab[i] != p)
-      if (++i == n_stn_tab) {
-#if DEBUG_INVALID
-	 fputs("Station ", stderr);
-	 fprint_prefix(stderr, stn->name);
-	 fputs(" not in table\n\n", stderr);
-#endif
-#if 0
-	 print_prefix(stn->name);
-	 printf(" used: %d colour %d\n",
-		(!!stn->leg[2])<<2 | (!!stn->leg[1])<<1 | (!!stn->leg[0]),
-		stn->colour);
-#endif
-	 fatalerror(/*Bug in program detected! Please report this to the authors*/11);
-      }
-   return i;
-}
-
-static int
-add_stn_to_tab(node *stn)
-{
-   int i;
-   pos *p = stn->name->pos;
-   for (i = 0; i < n_stn_tab; i++) {
-      if (stn_tab[i] == p) return i;
-   }
-   stn_tab[n_stn_tab++] = p;
-   return i;
-}
-
 /* Solve MX=B for X by Choleski factorisation - modified Choleski actually
  * since we factor into LDL' while Choleski is just LL'
  */
@@ -372,35 +348,33 @@ add_stn_to_tab(node *stn)
 static void
 choleski(real *M, real *B, long n)
 {
-   int i, j, k;
-
-   for (j = 1; j < n; j++) {
+   for (int j = 1; j < n; j++) {
       real V;
-      for (i = 0; i < j; i++) {
+      for (int i = 0; i < j; i++) {
 	 V = (real)0.0;
-	 for (k = 0; k < i; k++) V += M(i,k) * M(j,k) * M(k,k);
+	 for (int k = 0; k < i; k++) V += M(i,k) * M(j,k) * M(k,k);
 	 M(j,i) = (M(j,i) - V) / M(i,i);
       }
       V = (real)0.0;
-      for (k = 0; k < j; k++) V += M(j,k) * M(j,k) * M(k,k);
+      for (int k = 0; k < j; k++) V += M(j,k) * M(j,k) * M(k,k);
       M(j,j) -= V; /* may be best to add M() last for numerical reasons too */
    }
 
    /* Multiply x by L inverse */
-   for (i = 0; i < n - 1; i++) {
-      for (j = i + 1; j < n; j++) {
+   for (int i = 0; i < n - 1; i++) {
+      for (int j = i + 1; j < n; j++) {
 	 B[j] -= M(j,i) * B[i];
       }
    }
 
    /* Multiply x by D inverse */
-   for (i = 0; i < n; i++) {
+   for (int i = 0; i < n; i++) {
       B[i] /= M(i,i);
    }
 
    /* Multiply x by (L transpose) inverse */
-   for (i = (int)(n - 1); i > 0; i--) {
-      for (j = i - 1; j >= 0; j--) {
+   for (int i = (int)(n - 1); i > 0; i--) {
+      for (int j = i - 1; j >= 0; j--) {
 	 B[j] -= M(i,j) * B[i];
       }
    }
@@ -417,40 +391,39 @@ choleski(real *M, real *B, long n)
 static void
 sor(real *M, real *B, long n)
 {
-   real t, x, delta, threshold, t2;
-   int row, col;
-   real *X;
    long it = 0;
 
-   X = osmalloc(n * ossizeof(real));
+   real *X = osmalloc(n * ossizeof(real));
 
-   threshold = 0.00001;
+   const real threshold = 0.00001;
 
    printf("reciprocating diagonal\n"); /* TRANSLATE */
 
    /* munge diagonal so we can multiply rather than divide */
-   for (row = n - 1; row >= 0; row--) {
+   for (int row = n - 1; row >= 0; row--) {
       M(row,row) = 1 / M(row,row);
       X[row] = 0;
    }
 
    printf("starting iteration\n"); /* TRANSLATE */
 
+   real t;
    do {
       /*printf("*");*/
       it++;
       t = 0.0;
-      for (row = 0; row < n; row++) {
-	 x = B[row];
+      for (int row = 0; row < n; row++) {
+	 real x = B[row];
+	 int col;
 	 for (col = 0; col < row; col++) x -= M(row,col) * X[col];
 	 for (col++; col < n; col++) x -= M(col,row) * X[col];
 	 x *= M(row,row);
-	 delta = (x - X[row]) * SOR_factor;
-	 X[row] += delta;
-	 t2 = fabs(delta);
+	 real sor_delta = (x - X[row]) * SOR_factor;
+	 X[row] += sor_delta;
+	 real t2 = fabs(sor_delta);
 	 if (t2 > t) t = t2;
       }
-      printf("% 6d: %8.6f\n", it, t);
+      printf("% 6ld: %8.6f\n", it, t);
    } while (t >= threshold && it < 100000);
 
    if (t >= threshold) {
@@ -462,9 +435,9 @@ sor(real *M, real *B, long n)
 
 #if 0
    putnl();
-   for (row = n - 1; row >= 0; row--) {
+   for (int row = n - 1; row >= 0; row--) {
       t = 0.0;
-      for (col = 0; col < row; col++) t += M(row, col) * X[col];
+      for (int col = 0; col < row; col++) t += M(row, col) * X[col];
       t += X[row] / M(row, row);
       for (col = row + 1; col < n; col++)
 	 t += M(col, row) * X[col];
@@ -472,7 +445,7 @@ sor(real *M, real *B, long n)
    }
 #endif
 
-   for (row = n - 1; row >= 0; row--) B[row] = X[row];
+   for (int row = n - 1; row >= 0; row--) B[row] = X[row];
 
    osfree(X);
    printf("\ndone\n"); /* TRANSLATE */
@@ -483,9 +456,9 @@ sor(real *M, real *B, long n)
 static void
 print_matrix(real *M, real *B, long n)
 {
-   long row, col;
    printf("Matrix, M and vector, B:\n");
-   for (row = 0; row < n; row++) {
+   for (long row = 0; row < n; row++) {
+      long col;
       for (col = 0; col <= row; col++) printf("%6.2f\t", M(row, col));
       for (; col <= n; col++) printf(" \t");
       printf("\t%6.2f\n", B[row]);
