@@ -109,7 +109,7 @@ read_prefix(unsigned pfx_flags)
 	  (isSep(ch) || (pcs->dash_for_anon_wall_station && ch == '-'))) {
 	 int first_ch = ch;
 	 nextch();
-	 if (isBlank(ch) || isEol(ch)) {
+	 if (isBlank(ch) || isComm(ch) || isEol(ch)) {
 	    if (!isSep(first_ch))
 	       goto anon_wall_station;
 	    /* A single separator alone ('.' by default) is an anonymous
@@ -126,7 +126,7 @@ read_prefix(unsigned pfx_flags)
 	 }
 	 if (isSep(first_ch) && ch == first_ch) {
 	    nextch();
-	    if (isBlank(ch) || isEol(ch)) {
+	    if (isBlank(ch) || isComm(ch) || isEol(ch)) {
 	       /* A double separator ('..' by default) is an anonymous station
 		* which is on the wall and implies the leg to it is a splay.
 		*/
@@ -144,7 +144,7 @@ anon_wall_station:
 	    }
 	    if (ch == first_ch) {
 	       nextch();
-	       if (isBlank(ch) || isEol(ch)) {
+	       if (isBlank(ch) || isComm(ch) || isEol(ch)) {
 		  /* A triple separator ('...' by default) is an anonymous
 		   * station, but otherwise not handled specially (e.g. for
 		   * a single leg down an unexplored side passage to a station
@@ -376,8 +376,9 @@ read_walls_prefix(void)
 }
 
 prefix *
-read_walls_station(char * const walls_prefix[3], bool anon_allowed)
+read_walls_station(char * const walls_prefix[3], bool anon_allowed, bool *p_new)
 {
+    if (p_new) *p_new = false;
 //    bool f_optional = false; //!!(pfx_flags & PFX_OPT);
 //    bool fSuspectTypo = false; //!!(pfx_flags & PFX_SUSPECT_TYPO);
 //    prefix *back_ptr, *ptr;
@@ -416,6 +417,8 @@ read_walls_station(char * const walls_prefix[3], bool anon_allowed)
 	    pcs->flags |= BIT(FLAGS_ANON_ONE_END) | BIT(FLAGS_IMPLICIT_SPLAY);
 	    prefix *pfx = new_anon_station();
 	    pfx->sflags |= BIT(SFLAGS_WALL);
+	    // An anonymous station is always new.
+	    if (p_new) *p_new = true;
 	    return pfx;
 	}
 	s_catn(&component, dashes, '-');
@@ -452,10 +455,18 @@ read_walls_station(char * const walls_prefix[3], bool anon_allowed)
 
 	// component is the station name itself.
 	if (s_empty(&component)) {
-	    compile_diagnostic(DIAG_ERR|DIAG_COL, /*Expecting station name*/28);
-	    s_free(&component);
-	    for (int i = 0; i < 3; ++i) osfree(w_prefix[i]);
-	    LONGJMP(file.jbSkipLine);
+	    if (explicit_prefix_levels == 0) {
+		compile_diagnostic(DIAG_ERR|DIAG_COL, /*Expecting station name*/28);
+		s_free(&component);
+		for (int i = 0; i < 3; ++i) osfree(w_prefix[i]);
+		LONGJMP(file.jbSkipLine);
+	    }
+	    // Walls allows an empty station name if there's an explicit prefix.
+	    // This seems unlikely to be intended, so warn about it.
+	    compile_diagnostic(DIAG_WARN|DIAG_COL, /*Expecting station name*/28);
+	    // Use a name with a space in so it can't collide with a real
+	    // Walls station name.
+	    s_cat(&component, "empty name");
 	}
 	int len = s_len(&component);
 	char *p = s_steal(&component);
@@ -508,6 +519,9 @@ read_walls_station(char * const walls_prefix[3], bool anon_allowed)
 	    ptr = ptr->down;
 	    if (ptr == NULL) {
 		/* Special case first time around at each level */
+		/* No need to check if we're at the station level - if the
+		 * prefix is new the station must be. */
+		if (p_new) *p_new = true;
 		ptr = osnew(prefix);
 		ptr->ident = (i < 3 ? osstrdup(name) : name);
 		name = NULL;
@@ -537,8 +551,8 @@ read_walls_station(char * const walls_prefix[3], bool anon_allowed)
 		}
 		if (cmp) {
 		    /* ie we got to one that was higher, or the end */
-		    prefix *newptr;
-		    newptr = osnew(prefix);
+		    if (p_new) *p_new = true;
+		    prefix *newptr = osnew(prefix);
 		    newptr->ident = (i < 3 ? osstrdup(name) : name);
 		    name = NULL;
 		    if (ptrPrev == NULL)
@@ -557,9 +571,10 @@ read_walls_station(char * const walls_prefix[3], bool anon_allowed)
 		    newptr->sflags = sflag;
 		    ptr = newptr;
 		} else {
-		    if ((ptr->sflags & sflag) == 0) {
-			// FIXME diagnostic?
-		    }
+		    ptr->sflags |= sflag;
+		}
+		if (!TSTBIT(ptr->sflags, SFLAGS_SURVEY)) {
+		    ptr->min_export = USHRT_MAX;
 		}
 		cached_survey = back_ptr;
 		cached_station = ptr;
@@ -567,7 +582,12 @@ read_walls_station(char * const walls_prefix[3], bool anon_allowed)
 	    if (name == p) osfree(p);
 	}
 
-	// fprint_prefix(stdout, ptr); fputnl(stdout);
+	// Do the equivalent of "*infer exports" for Walls stations with an
+	// explicit prefix.
+	if (ptr->min_export == 0 || ptr->min_export == USHRT_MAX) {
+	    if (explicit_prefix_levels > ptr->max_export)
+		ptr->max_export = explicit_prefix_levels;
+	}
 
 	for (int i = 0; i < 3; ++i) osfree(w_prefix[i]);
 
@@ -654,7 +674,7 @@ read_quadrant(bool f_optional)
    const int quad = 90;
    filepos fp;
    get_pos(&fp);
-   get_token_no_blanks();
+   get_token_legacy_no_blanks();
    int first_point = match_tok(pointtab, TABSIZE(pointtab));
    if (first_point == POINT_NONE) {
       set_pos(&fp);
@@ -684,7 +704,7 @@ read_quadrant(bool f_optional)
       return 0.0; /* for brain-fried compilers */
    }
 
-   get_token_no_blanks();
+   get_token_legacy_no_blanks();
    int second_point = match_tok(pointewtab, TABSIZE(pointewtab));
    if (second_point == POINT_NONE) {
       set_pos(&fp);
@@ -695,7 +715,7 @@ read_quadrant(bool f_optional)
 
    if (r > quad) {
       set_pos(&fp);
-      compile_diagnostic_token_show(DIAG_ERR|DIAG_COL, /*Suspicious compass reading*/59);
+      compile_diagnostic_token_show(DIAG_ERR, /*Suspicious compass reading*/59);
       LONGJMP(file.jbSkipLine);
       return 0.0; /* for brain-fried compilers */
    }
@@ -730,10 +750,13 @@ read_numeric_multi(bool f_optional, bool f_quadrants, int *p_n_readings)
    skipblanks();
    if (!isOpen(ch)) {
       real r = 0;
-      if (!f_quadrants)
+      if (!f_quadrants) {
 	  r = read_number(f_optional, false);
-      else
+      } else {
 	  r = read_quadrant(f_optional);
+	  if (r != HUGE_REAL)
+	      do_legacy_token_warning();
+      }
       if (p_n_readings) *p_n_readings = (r == HUGE_REAL ? 0 : 1);
       return r;
    }
@@ -741,10 +764,12 @@ read_numeric_multi(bool f_optional, bool f_quadrants, int *p_n_readings)
 
    skipblanks();
    do {
-      if (!f_quadrants)
+      if (!f_quadrants) {
 	 tot += read_number(false, false);
-      else
+      } else {
 	 tot += read_quadrant(false);
+	 do_legacy_token_warning();
+      }
       ++n_readings;
       skipblanks();
    } while (!isClose(ch));
